@@ -5,6 +5,7 @@
    Copyright (C) 2005 Paul Smedley <paul@smedley.info> (OS/2 usbcalls)
    Copyright (C) 2008 m. allan noah (bus rescan support, sanei_usb_clear_halt)
    Copyright (C) 2009 Julien BLACHE <jb@jblache.org> (libusb-1.0)
+   Copyright (C) 2011 Reinhold Kainhofer <reinhold@kainhofer.com> (sanei_usb_set_endpoint)
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -1104,6 +1105,77 @@ sanei_usb_find_devices (SANE_Int vendor, SANE_Int product,
   return SANE_STATUS_GOOD;
 }
 
+void
+sanei_usb_set_endpoint (SANE_Int dn, SANE_Int ep_type, SANE_Int ep)
+{
+  if (dn >= device_number || dn < 0)
+    {
+      DBG (1, "sanei_usb_set_endpoint: dn >= device number || dn < 0\n");
+      return;
+    }
+
+  DBG (5, "sanei_usb_set_endpoint: Setting endpoint of type 0x%02x to 0x%02x\n", ep_type, ep);
+  switch (ep_type)
+    {
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_BULK:
+	    devices[dn].bulk_in_ep  = ep;
+	    break;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_BULK:
+	    devices[dn].bulk_out_ep = ep;
+	    break;
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_ISOCHRONOUS:
+	    devices[dn].iso_in_ep = ep;
+	    break;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_ISOCHRONOUS:
+	    devices[dn].iso_out_ep = ep;
+	    break;
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_INTERRUPT:
+	    devices[dn].int_in_ep = ep;
+	    break;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_INTERRUPT:
+	    devices[dn].int_out_ep = ep;
+	    break;
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_CONTROL:
+	    devices[dn].control_in_ep = ep;
+	    break;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_CONTROL:
+	    devices[dn].control_out_ep = ep;
+	    break;
+    }
+}
+
+SANE_Int
+sanei_usb_get_endpoint (SANE_Int dn, SANE_Int ep_type)
+{
+  if (dn >= device_number || dn < 0)
+    {
+      DBG (1, "sanei_usb_get_endpoint: dn >= device number || dn < 0\n");
+      return 0;
+    }
+
+  switch (ep_type)
+    {
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_BULK:
+	    return devices[dn].bulk_in_ep;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_BULK:
+	    return devices[dn].bulk_out_ep;
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_ISOCHRONOUS:
+	    return devices[dn].iso_in_ep;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_ISOCHRONOUS:
+	    return devices[dn].iso_out_ep;
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_INTERRUPT:
+	    return devices[dn].int_in_ep;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_INTERRUPT:
+	    return devices[dn].int_out_ep;
+      case USB_DIR_IN|USB_ENDPOINT_TYPE_CONTROL:
+	    return devices[dn].control_in_ep;
+      case USB_DIR_OUT|USB_ENDPOINT_TYPE_CONTROL:
+	    return devices[dn].control_out_ep;
+      default:
+	    return 0;
+    }
+}
+
 SANE_Status
 sanei_usb_open (SANE_String_Const devname, SANE_Int * dn)
 {
@@ -1977,6 +2049,12 @@ sanei_usb_clear_halt (SANE_Int dn)
 #ifdef HAVE_LIBUSB
   int ret;
 
+  if (dn >= device_number || dn < 0)
+    {
+      DBG (1, "sanei_usb_clear_halt: dn >= device number || dn < 0\n");
+      return SANE_STATUS_INVAL;
+    }
+
   ret = usb_clear_halt (devices[dn].libusb_handle, devices[dn].bulk_in_ep);
   if (ret){
     DBG (1, "sanei_usb_clear_halt: BULK_IN ret=%d\n", ret);
@@ -1996,6 +2074,12 @@ sanei_usb_clear_halt (SANE_Int dn)
 #elif defined(HAVE_LIBUSB_1_0)
   int ret;
 
+  if (dn >= device_number || dn < 0)
+    {
+      DBG (1, "sanei_usb_clear_halt: dn >= device number || dn < 0\n");
+      return SANE_STATUS_INVAL;
+    }
+
   ret = libusb_clear_halt (devices[dn].lu_handle, devices[dn].bulk_in_ep);
   if (ret){
     DBG (1, "sanei_usb_clear_halt: BULK_IN ret=%d\n", ret);
@@ -2014,6 +2098,34 @@ sanei_usb_clear_halt (SANE_Int dn)
   return SANE_STATUS_GOOD;
 }
 
+SANE_Status
+sanei_usb_reset (SANE_Int dn)
+{
+#ifdef HAVE_LIBUSB
+  int ret;
+
+  ret = usb_reset (devices[dn].libusb_handle);
+  if (ret){
+    DBG (1, "sanei_usb_reset: ret=%d\n", ret);
+    return SANE_STATUS_INVAL;
+  }
+
+#elif defined(HAVE_LIBUSB_1_0)
+  int ret;
+
+  ret = libusb_reset_device (devices[dn].lu_handle);
+  if (ret){
+    DBG (1, "sanei_usb_reset: ret=%d\n", ret);
+    return SANE_STATUS_INVAL;
+  }
+  
+#else /* not HAVE_LIBUSB && not HAVE_LIBUSB_1_0 */
+  DBG (1, "sanei_usb_reset: libusb support missing\n");
+#endif /* HAVE_LIBUSB || HAVE_LIBUSB_1_0 */
+
+  return SANE_STATUS_GOOD;
+}
+
 SANE_Status
 sanei_usb_read_bulk (SANE_Int dn, SANE_Byte * buffer, size_t * size)
 {
@@ -2262,7 +2374,8 @@ sanei_usb_write_bulk (SANE_Int dn, const SANE_Byte * buffer, size_t * size)
       if (devices[dn].bulk_out_ep){
         rc = UsbBulkWrite (dh, devices[dn].bulk_out_ep, devices[dn].interface_nr,
                                ulToWrite, (char*) buffer, usbcalls_timeout);
-        DBG (1, "sanei_usb_write_bulk: rc = %d\n",rc);}
+        DBG (1, "sanei_usb_write_bulk: rc = %d\n",rc);
+      }
       else
       {
           DBG (1, "sanei_usb_write_bulk: can't read without a bulk-out endpoint\n");
